/*
 * Sonar, open source software quality management tool.
 * Copyright (C) 2009 SonarSource SA
 * mailto:contact AT sonarsource DOT com
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.dev;

import org.apache.maven.artifact.Artifact;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.plugin.MojoFailureException;

import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * Check that:
 * <ul>
 * <li>sonar-plugin-api declared in project dependenecies, otherwise fails;</li>
 * <li>sonar-plugin-api declared with 'provided' scope, otherwise fails;</li>
 * <li>any plugin dependencies are specified with 'provided' scope, otherwise show warning.</li>
 * </ul>
 *
 * @author Evgeny Mandrikov
 * @goal check-plugin-dependencies
 * @requiresDependencyResolution runtime
 * @phase initialize
 */
public class ValidateDependenciesMojo extends AbstractSonarPluginMojo {
  public void execute() throws MojoExecutionException, MojoFailureException {
    // Check sonar-plugin-api
    Artifact sonarPluginApi = getSonarPluginApiArtifact();
    if (sonarPluginApi == null) {
      throw new MojoExecutionException(
          SONAR_PLUGIN_API_GROUPID + ":" + SONAR_PLUGIN_API_ARTIFACTID + " should be declared in dependencies"
      );
    } else {
      if (!Artifact.SCOPE_PROVIDED.equals(sonarPluginApi.getScope())) {
        throw new MojoExecutionException(
            SONAR_PLUGIN_API_GROUPID + ":" + SONAR_PLUGIN_API_ARTIFACTID + " should be declared with 'provided' scope"
        );
      }
    }
    // Check scope of dependencies
    Set<Artifact> dependencies = getDependencyArtifacts();
    List<String> failures = new ArrayList<String>();
    for (Artifact dep : dependencies) {
      if (Artifact.SCOPE_PROVIDED.equals(dep.getScope())) {
        getLog().info("Found dependency with 'provided' scope: " + dep.getDependencyConflictId() + " - ignoring");
        continue;
      } else if (Artifact.SCOPE_TEST.equals(dep.getScope())) {
        getLog().info("Found dependency with 'test' scope: " + dep.getDependencyConflictId() + " - ignoring");
        continue;
      }
      failures.add(dep.getId());
    }
    if (!failures.isEmpty()) {
      StringBuilder message = new StringBuilder();
      message.append("Found following dependencies without 'provided' scope:\n");
      for (String id : failures) {
        message.append("\n - ").append(id);
      }
      message.append('\n')
          .append("\nSee following page for more details about plugin dependencies:")
          .append("\n    http://docs.codehaus.org/display/SONAR/Developing+a+plugin#Developingaplugin-Dependencymanagement")
          .append('\n');
      getLog().warn(message);
    } else {
      getLog().info("All plugin dependencies in this project seem to be correct.");
    }
  }
}
